#include "createfiledialog.h"
#include "archiver.h"
#include "ui_create.h"

#include <QBoxLayout>
#include <QSettings>
#include <QScreen>
#include <QDesktopWidget>
#include <QApplication>
#include <QMimeDatabase>
#include <QTranslator>

CreateFileDialog::CreateFileDialog(QWidget *parent, Fm::FilePath path):
    Fm::FileDialog{parent, std::move(path)},
    ui_{new Ui::CreateArchiveExtraWidget{}}
{
    QIcon icon;
    QString iconThemeName = QStringLiteral("nde-archiver");
    if (QIcon::hasThemeIcon(iconThemeName)) {
        icon = QIcon::fromTheme(iconThemeName);
    } else {
        icon.addFile(QStringLiteral("."), QSize(), QIcon::Normal, QIcon::Off);
    }
    setWindowIcon(icon);

    setAcceptMode(QFileDialog::AcceptSave);
    setTypeFilterEnable(false);
    connect(this, &Fm::FileDialog::filterSelected, this, &CreateFileDialog::typeChanged);

    QStringList typeList;
    const auto mimeTypes = Archiver::supportedCreateMimeTypes();

    if(!mimeTypes.isEmpty()) {
        for (int i = 0; i< mimeTypes.count(); i++){
            QMimeDatabase mimeDb;
            auto mimeType = mimeDb.mimeTypeForName(mimeTypes.at(i));
            if(mimeType.isValid()) {
                auto suffixes = mimeType.suffixes();
                if(!suffixes.isEmpty()) {
                    typeList  << suffixes.at(0);
                }
            }
        }
    }

    setNameFilters(typeList);
    currentType = typeList.at(0);

    // extra options
    auto extraWidget = new QWidget{this};
    ui_->setupUi(extraWidget);
    auto boxLayout = qobject_cast<QBoxLayout*>(layout());
    if(boxLayout) {
        boxLayout->addWidget(extraWidget);
    }

    // default to 10 MB
    ui_->volumeSize->setValue(10);

    ui_->label->hide();
    ui_->password->hide();
    ui_->encryptFileList->hide();
    ui_->splitVolumes->hide();
    ui_->volumeSize->hide();


    QDesktopWidget *desk = QApplication::desktop();
    QRect deskRect = desk->availableGeometry();
    this->move((deskRect.width()-this->width())/2,(deskRect.height()-this->height())/2);
}

CreateFileDialog::~CreateFileDialog() {
}

void CreateFileDialog::setTitle(QString str)
{
    setWindowTitle(str);
}

QString CreateFileDialog::password() const {
    return ui_->password->text();
}

bool CreateFileDialog::encryptFileList() const {
    return ui_->encryptFileList->isChecked();
}

bool CreateFileDialog::splitVolumes() const {
    return ui_->splitVolumes->isChecked();
}

unsigned int CreateFileDialog::volumeSize() const {
    return ui_->volumeSize->value()*1024*1024;
}

void CreateFileDialog::typeChanged(QString type){
    currentType = type;

    if (ui_->password && ui_->encryptFileList && ui_->splitVolumes && ui_->volumeSize){
        if ( currentType == "7z" || currentType == "rar"){
            ui_->label->show();
            ui_->password->show();
            ui_->encryptFileList->show();
            ui_->splitVolumes->show();
            ui_->volumeSize->show();
        }else{
            ui_->label->hide();
            ui_->password->hide();
            ui_->encryptFileList->hide();
            ui_->splitVolumes->hide();
            ui_->volumeSize->hide();
        }
    }

}
