#include "extractfiledialog.h"
#include "ui_extract.h"

#include <QBoxLayout>
#include <QSettings>
#include <QScreen>
#include <QDesktopWidget>
#include <QApplication>
#include <QTranslator>

ExtractFileDialog::ExtractFileDialog(QWidget *parent, Fm::FilePath path):
    Fm::FileDialog{parent, std::move(path)},
    ui_{new Ui::ExtractArchiveExtraWidget{}} {

    QIcon icon;
    QString iconThemeName = QStringLiteral("nde-archiver");
    if (QIcon::hasThemeIcon(iconThemeName)) {
        icon = QIcon::fromTheme(iconThemeName);
    } else {
        icon.addFile(QStringLiteral("."), QSize(), QIcon::Normal, QIcon::Off);
    }
    setWindowIcon(icon);

    setOptions(QFileDialog::ShowDirsOnly | QFileDialog::HideNameFilterDetails);
    setAcceptMode(QFileDialog::AcceptOpen);
    setFileMode(QFileDialog::Directory);

    // add extra options
    QWidget* extraWidget = new QWidget(this);
    ui_->setupUi(extraWidget);
    auto boxLayout = qobject_cast<QBoxLayout*>(layout());
    if(boxLayout) {
        boxLayout->addWidget(extraWidget);
    }

    QDesktopWidget *desk = QApplication::desktop();
    QRect deskRect = desk->availableGeometry();
    this->move((deskRect.width()-this->width())/2,(deskRect.height()-this->height())/2);
}

ExtractFileDialog::~ExtractFileDialog() {
}

void ExtractFileDialog::setTitle(QString str)
{
    setWindowTitle(str);
}

void ExtractFileDialog::setFilter(QStringList srtList)
{
    setNameFilters(srtList);
}

bool ExtractFileDialog::skipOlder() const {
    return ui_->skipOlder->isChecked();
}

bool ExtractFileDialog::overwrite() const {
    return ui_->overwriteExisting->isChecked();
}

bool ExtractFileDialog::reCreateFolders() const {
    return ui_->reCreateFolders->isChecked();
}

bool ExtractFileDialog::extractAll() const {
    return ui_->extractAll->isChecked();
}

void ExtractFileDialog::setExtractAll(bool value){
    ui_->extractAll->setChecked(value);
}

void ExtractFileDialog::setExtractSelected(bool value){
    ui_->extractSelected->setChecked(value);
}

void ExtractFileDialog::setExtractAllEnabled(bool enabled)
{
    ui_->extractAll->setEnabled(enabled);
}

void ExtractFileDialog::setExtractSelectedEnabled(bool enabled) {
    ui_->extractSelected->setEnabled(enabled);
}
